//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : CPUTime
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "cpu_time.h"

//============================================================================//
void CPUTime::initialize()
//============================================================================//
{

    t1_initialize = 0.0;
    t2_receptor_process = 0.0;
    t3_docking_total = 0.0;
    t4_docking_output_detail = 0.0;
    t5_docking_output = 0.0;

    t3_1_ligand_voxelization = 0.0;
    t3_2_fftprocess_ligand_fft = 0.0;
    t3_3_fftprocess_modulation = 0.0;
    t3_4_fftprocess_fft_inverse = 0.0;
    t3_5_fftprocess_score_sort = 0.0;
    t3_6_fftprocess_sort_index = 0.0;

    t3_1_1_ligvoxgpu_copy_htod = 0.0;
    t3_1_2_ligvoxgpu_kernel_init = 0.0;
    t3_1_3_ligvoxgpu_kernel_fill_core = 0.0;
    t3_1_4_ligvoxgpu_kernel_cut_surf = 0.0;
    t3_1_5_ligvoxgpu_kernel_fill_surf = 0.0;
    t3_1_6_ligvoxgpu_kernel_elec = 0.0;
    t3_1_7_ligvoxgpu_kernel_set_array = 0.0;
    
    t4_data_transfer_rec = 0.0;
	t4_data_transfer_lig = 0.0;
	t4_data_transfer_in_loop = 0.0;

	m1_current_malloc_size = 0;
	m2_maximum_malloc_size = 0;

    return;
}

//============================================================================//
void CPUTime::output()
//============================================================================//
{

    float total_time =  t1_initialize+
                        t2_receptor_process+
                        t3_docking_total+
                        t4_docking_output_detail+
                        t5_docking_output;

    float total_transfer =  
    				    t4_data_transfer_rec+
                        t4_data_transfer_lig+
                        t4_data_transfer_in_loop;
	/*
    printf("CPU time (data_transfer_rec     = %8.2f sec. \n",t4_data_transfer_rec     );
    printf("CPU time (data_transfer_lig     = %8.2f sec. \n",t4_data_transfer_lig     );
    printf("CPU time (data_transfer_in_loop = %8.2f sec. \n",t4_data_transfer_in_loop );
    printf("CPU time (total_transfer        = %8.2f sec. \n",total_transfer           );
    //*/


    //correct the calc-time gap due to parallel calc
    const float cufft_ligvox_on_GPU_total_idzero = 0.00000000001
    	    + t3_1_1_ligvoxgpu_copy_htod
            + t3_1_2_ligvoxgpu_kernel_init
            + t3_1_3_ligvoxgpu_kernel_fill_core
            + t3_1_4_ligvoxgpu_kernel_cut_surf
            + t3_1_5_ligvoxgpu_kernel_fill_surf
            + t3_1_6_ligvoxgpu_kernel_elec
            + t3_1_7_ligvoxgpu_kernel_set_array;
    //printf(" max:%f, one:%f",t3_1_ligand_voxelization,cufft_ligvox_on_GPU_total_idzero);
    t3_1_1_ligvoxgpu_copy_htod            *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    t3_1_2_ligvoxgpu_kernel_init          *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    t3_1_3_ligvoxgpu_kernel_fill_core     *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    t3_1_4_ligvoxgpu_kernel_cut_surf      *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    t3_1_5_ligvoxgpu_kernel_fill_surf     *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    t3_1_6_ligvoxgpu_kernel_elec          *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    t3_1_7_ligvoxgpu_kernel_set_array     *= (t3_1_ligand_voxelization / cufft_ligvox_on_GPU_total_idzero);
    
    const float t3_docking_total_idzero = 0.00000000001
    								   + t3_1_ligand_voxelization
                                       + t3_2_fftprocess_ligand_fft
                                       + t3_3_fftprocess_modulation
                                       + t3_4_fftprocess_fft_inverse
                                       + t3_5_fftprocess_score_sort
                                       + t3_6_fftprocess_sort_index;
    //printf(" max:%f, one:%f",t3_docking_total,t3_docking_total_idzero);
    t3_1_ligand_voxelization     *= (t3_docking_total / t3_docking_total_idzero);
    t3_2_fftprocess_ligand_fft   *= (t3_docking_total / t3_docking_total_idzero);
    t3_3_fftprocess_modulation   *= (t3_docking_total / t3_docking_total_idzero);
    t3_4_fftprocess_fft_inverse  *= (t3_docking_total / t3_docking_total_idzero);
    t3_5_fftprocess_score_sort   *= (t3_docking_total / t3_docking_total_idzero);
    t3_6_fftprocess_sort_index   *= (t3_docking_total / t3_docking_total_idzero);

    printf("\n");
    printf("CPU time (initialize)         = %8.2f sec. (%4.1f%%)\n",t1_initialize                     , 100.0*t1_initialize                     /total_time);
    printf("CPU time (receptor process)   = %8.2f sec. (%4.1f%%)\n",t2_receptor_process               , 100.0*t2_receptor_process               /total_time);
    printf("CPU time (docking)            = %8.2f sec. (%4.1f%%)\n",t3_docking_total                  , 100.0*t3_docking_total                  /total_time);
    printf("  | Ligand voxelization       = %8.2f sec. (%4.1f%%)\n",t3_1_ligand_voxelization          , 100.0*t3_1_ligand_voxelization          /total_time);
#ifdef CUFFT
/*
if(cufft_ligvox_on_GPU_total_idzero>0.1){
    printf("    | ligvox_copy_htod        = %8.2f sec. (%4.1f%%)\n",t3_1_1_ligvoxgpu_copy_htod        , 100.0*t3_1_1_ligvoxgpu_copy_htod        /total_time);
    printf("    | ligvox_kernel_init      = %8.2f sec. (%4.1f%%)\n",t3_1_2_ligvoxgpu_kernel_init      , 100.0*t3_1_2_ligvoxgpu_kernel_init      /total_time);
    printf("    | ligvox_kernel_fill_core = %8.2f sec. (%4.1f%%)\n",t3_1_3_ligvoxgpu_kernel_fill_core , 100.0*t3_1_3_ligvoxgpu_kernel_fill_core /total_time);
    printf("    | ligvox_kernel_cut_surf  = %8.2f sec. (%4.1f%%)\n",t3_1_4_ligvoxgpu_kernel_cut_surf  , 100.0*t3_1_4_ligvoxgpu_kernel_cut_surf  /total_time);
    printf("    | ligvox_kernel_fill_surf = %8.2f sec. (%4.1f%%)\n",t3_1_5_ligvoxgpu_kernel_fill_surf , 100.0*t3_1_5_ligvoxgpu_kernel_fill_surf /total_time);
    printf("    | ligvox_kernel_elec      = %8.2f sec. (%4.1f%%)\n",t3_1_6_ligvoxgpu_kernel_elec      , 100.0*t3_1_6_ligvoxgpu_kernel_elec      /total_time);
    printf("    | ligvox_kernel_set_array = %8.2f sec. (%4.1f%%)\n",t3_1_7_ligvoxgpu_kernel_set_array , 100.0*t3_1_7_ligvoxgpu_kernel_set_array /total_time);
}
//*/
#endif
    printf("  | Ligand FFT                = %8.2f sec. (%4.1f%%)\n",t3_2_fftprocess_ligand_fft        , 100.0*t3_2_fftprocess_ligand_fft        /total_time);
    printf("  | Modulation                = %8.2f sec. (%4.1f%%)\n",t3_3_fftprocess_modulation        , 100.0*t3_3_fftprocess_modulation        /total_time);
    printf("  | Inverse FFT to get score  = %8.2f sec. (%4.1f%%)\n",t3_4_fftprocess_fft_inverse       , 100.0*t3_4_fftprocess_fft_inverse       /total_time);
    printf("  | Sort score (each core)    = %8.2f sec. (%4.1f%%)\n",t3_5_fftprocess_score_sort        , 100.0*t3_5_fftprocess_score_sort        /total_time);
    printf("  | Sort score (merge)        = %8.2f sec. (%4.1f%%)\n",t3_6_fftprocess_sort_index        , 100.0*t3_6_fftprocess_sort_index        /total_time);
    printf("CPU time (detailed output)    = %8.2f sec. (%4.1f%%)\n",t4_docking_output_detail          , 100.0*t4_docking_output_detail          /total_time);
    printf("CPU time (output)             = %8.2f sec. (%4.1f%%)\n",t5_docking_output                 , 100.0*t5_docking_output                 /total_time);

    return;
}
