//============================================================================//
//
//  Software Name : MEGADOCK
//
//  Class Name : Ligand
//
//  Contact address : Tokyo Institute of Technology, AKIYAMA Lab.
//
//============================================================================//

#include "ligand.h"
#include <algorithm> // shimoda

//============================================================================//
void Ligand::rpscace(const float &aceratio, const int &num_grid,float *grid_coord,
                     float *mol_coord, int *iwork, float *fwork,
                     const float &param_rec_core, const float &param_lig_core,const int &old_voxel_flag)
//============================================================================//
{
    const float delta = param_lig_core;       // Tuning Parameter
    const float surface = 1.0;        // grid-assignment score (protein surface)
    const float swollen_surface = -8888.0; // temporary score for swollen ligand surface (121004 shimoda bugfix)
    const float grid_width = 1.2;

    int       *nearesta;
    int       *surf_atom;
    float     *grid_r, *grid_i;
    float     *rjudge2, *radius_core2, *radius_surf2;
    float     *distance3;
    float     *xd, *yd, *zd;

    const int ng1 = num_grid;
    const int ng2 = ng1 * ng1;
    const int ng3 = ng1 * ng1 * ng1;
    const int na  = num_atoms();
    const int nag = num_atoms() * ng1;

    nearesta  = iwork;
    surf_atom = &iwork[ng3];
    grid_r    = fwork;
    grid_i    = &fwork[ng3];
    rjudge2   = &fwork[ng3*2];
    radius_core2  = &fwork[ng3*2+na];
    radius_surf2  = &fwork[ng3*2+na*2];
    distance3     = &fwork[ng3*2+na*3];
    xd        = &fwork[ng3*3+na*3];
    yd        = &fwork[ng3*3+na*3+nag];
    zd        = &fwork[ng3*3+na*3+nag*2];

    precalc(ng1,nearesta,rjudge2,radius_core2,radius_surf2);
    voxel_init(ng1,grid_coord,mol_coord,grid_r,grid_i,distance3,xd,yd,zd);

    if(old_voxel_flag==0) {

//#ifdef NEWVOXEL_rPSC

// ##############################
// #fill up grid by atom-sphere #
// ##############################
//
// Reason for search_range = 2
//   search_range means the range of searching grids(around atom)
//   If search_range = 2 and nearest grid is (10,5,-2), search from (8,3,-4) to (12,7,0) as rectangular
//   In the 3 next grid, distance between grid and atom is 3A (2*1.2A + 1.2A/2)
//   2 is enough because max VdW radius (charmm) is 2.13A and rcore2 = 1.5, so 2.13A * sqrt(1.5) = 2.61A < 3A

        //*
        for( int l = 0 ; l < na ; l++ ) {// atom-grid assignment
            const int search_range = 2;
            const int lc = ng1 * l;
            const int i2 = mol_coord[3*l  ] / grid_width + ng1 / 2;
            const int j2 = mol_coord[3*l+1] / grid_width + ng1 / 2;
            const int k2 = mol_coord[3*l+2] / grid_width + ng1 / 2;
            const int ia = max(i2 - search_range, 0);
            const int ja = max(j2 - search_range, 0);
            const int ka = max(k2 - search_range, 0);
            const int ib = min(i2 + search_range+1, ng1);
            const int jb = min(j2 + search_range+1, ng1);
            const int kb = min(k2 + search_range+1, ng1);

            for( int i = ia ; i < ib ; i++ ) {// grid around atom[l]
                if(xd[lc+i] > radius_core2[l]) continue;

                for( int j = ja ; j < jb ; j++ ) {
                    const float d2 = xd[lc+i]+yd[lc+j];
                    const int ij=ng2*i+ng1*j;
                    if(d2 > radius_core2[l]) continue;

                    for( int k = ka ; k < kb ; k++ ) {
                        const float d3 = xd[lc+i] + yd[lc+j] + zd[lc+k];
                        if( d3 < radius_core2[l] ) {// distance(grid-atom) < Van der Waals radius (* core)

                            grid_r[ij+k] = delta;    // grid[i] is filled up by atom[l]

                        }
                    }
                }
            }
        }


// #####################
// #fill up cavity     #
// #####################
//
// This part is comment out
// because this part is time-consuming and does not have clear contribution to result

        //if(_parameter->_grid_fillup_ligand == 1) { //parameter option

        /*
        for (int ij=0; ij<ng2; ij++) {// fill up cavity (phase1, 1/2)
            const int i = ij % ng1;
            const int j = ij / ng1;

            for (int k=0; k<ng1; k++) {// X, forward
                const int kji = i+ng1*j+ng2*k;
                if(grid_r[kji]!=delta) {
                    grid_r[kji]=open_space;
                } else {
                    break;
                }
            }

            for (int k=ng1-1; k>=0; k--) {// X, backward
                const int kji = i+ng1*j+ng2*k;
                if(grid_r[kji]!=delta && grid_r[kji]!=open_space) { //this cond is available only in X backward
                    grid_r[kji]=open_space;
                } else {
                    break;
                }
            }

            for (int k=0; k<ng1; k++) {// Y, forward
                const int jik = k+ng1*i+ng2*j;
                if(grid_r[jik]!=delta) {
                    grid_r[jik]=open_space;
                } else {
                    break;
                }
            }

            for (int k=ng1-1; k>=0; k--) {// Y, backward
                const int jik = k+ng1*i+ng2*j;
                if(grid_r[jik]!=delta) {
                    grid_r[jik]=open_space;
                } else {
                    break;
                }
            }

            for (int k=0; k<ng1; k++) {// Z, forward
                const int ikj = j+ng1*k+ng2*i;
                if(grid_r[ikj]!=delta) {
                    grid_r[ikj]=open_space;
                } else {
                    break;
                }
            }

            for (int k=ng1-1; k>=0; k--) {// Z, backward
                const int ikj = j+ng1*k+ng2*i;
                if(grid_r[ikj]!=delta) {
                    grid_r[ikj]=open_space;
                } else {
                    break;
                }
            }
        }
        //*/

        /*
        for (int ink=0; ink<2; ink++) {// fill up cavity (phase1, 2/2)

            for (int ijk=ng2; ijk<ng3-ng2; ijk++) {
                const int i = ijk / ng2;
                const int j = (ijk / ng1) % ng1;
                if(j<2||j>ng1-2) continue; // skip border
                const int k = ijk % ng1;
                if(k<2||k>ng1-2) continue; // skip border

                if(grid_r[ijk]==open_space) {
                    if(grid_r[ijk+1]==0) {
                        grid_r[ijk+1]=open_space;
                    }
                    if(grid_r[ijk-1]==0) {
                        grid_r[ijk-1]=open_space;
                    }
                    if(grid_r[ijk+ng1]==0) {
                        grid_r[ijk+ng1]=open_space;
                    }
                    if(grid_r[ijk-ng1]==0) {
                        grid_r[ijk-ng1]=open_space;
                    }
                    if(grid_r[ijk+ng2]==0) {
                        grid_r[ijk+ng2]=open_space;
                    }
                    if(grid_r[ijk-ng2]==0) {
                        grid_r[ijk-ng2]=open_space;
                    }
                }
            }
        }
        //*/

        /*
        for(int i=0; i<ng3; i++) {// fill up cavity (phase2)
            if(grid_r[i]==0) {
                grid_r[i]=delta;
            }
        }

        for(int i=0; i<ng3; i++) {// restore open_space to 0 (phase3)
            if(grid_r[i]==open_space) {
                grid_r[i]=0;
            }
        }
        //*/


// ############################
// # scrape swollen surface   #
// ############################
//
// 1. scrape surface grid (1 or 2 layer) because protein surface is swollen due to radius_core2
// 2. re-assign score by radius_surf2 in [make protein surface]

        for(int ijk=ng2; ijk<ng3-ng2; ijk++) {// convert surface grid (next to open space) to swollen_surface
            const int i = ijk / ng2;
            const int j = (ijk / ng1) % ng1;
            const int k = ijk % ng1;

            if(i==0||i==ng1-1||j==0||j==ng1-1||k==0||k==ng1-1)continue; // skip border

            if(grid_r[ijk]==delta) {
                if(grid_r[ijk-1]==0 ||
                        grid_r[ijk+1]==0 ||
                        grid_r[ijk-ng1]==0 ||
                        grid_r[ijk+ng1]==0 ||
                        grid_r[ijk-ng2]==0 ||
                        grid_r[ijk+ng2]==0) {
                    grid_r[ijk]=swollen_surface; // (121004 shimoda bugfix)
                }
            }
        }

        for(int ijk=ng2; ijk<ng3-ng2; ijk++) {
            if(grid_r[ijk]==swollen_surface) { // (121004 shimoda bugfix)
                grid_r[ijk]=0.0;
            }
        }

// ##############################
// #make protein surface        #
// ##############################

        for( int l = 0 ; l < na ; l++ ) {// for each atom
            const int search_range = 2;
            const int lc = ng1 * l;
            const int i2 = mol_coord[3*l  ] / grid_width + ng1 / 2;
            const int j2 = mol_coord[3*l+1] / grid_width + ng1 / 2;
            const int k2 = mol_coord[3*l+2] / grid_width + ng1 / 2;
            const int ia = max(i2 - search_range, 0);
            const int ja = max(j2 - search_range, 0);
            const int ka = max(k2 - search_range, 0);
            const int ib = min(i2 + search_range+1, ng1);
            const int jb = min(j2 + search_range+1, ng1);
            const int kb = min(k2 + search_range+1, ng1);

            for( int i = ia ; i < ib ; i++ ) {// grid around atom[l]
                if(xd[lc+i] > radius_surf2[l]) continue;

                for( int j = ja ; j < jb ; j++ ) {
                    const float d2 = xd[lc+i]+yd[lc+j];
                    const int ij=ng2*i+ng1*j;
                    if(d2 > radius_surf2[l]) continue;

                    for( int k = ka ; k < kb ; k++ ) {
                        const int ijk = ij+k;
                        if(grid_r[ijk]==delta) continue;

                        const float d3 = xd[lc+i] + yd[lc+j] + zd[lc+k];
                        if( d3 < radius_surf2[l] ) {// distance(grid-atom) < Van der Waals radius (* surf)
                            grid_r[ijk] = surface;    // grid[i] is filled up by atom[l]
                        }
                    }
                }
            }
        }
        //*/

        if(surface!=delta) {// convert surface grid (next to open space) to [1]
            for(int ijk=ng2; ijk<ng3-ng2; ijk++) {
                const int i = ijk / ng2;
                const int j = (ijk / ng1) % ng1;
                const int k = ijk % ng1;

                if(i==0||i==ng1-1||j==0||j==ng1-1||k==0||k==ng1-1)continue; // skip border

                if(grid_r[ijk]==delta) {
                    //*
                    if(grid_r[ijk-1]==0 ||
                            grid_r[ijk+1]==0 ||
                            grid_r[ijk-ng1]==0 ||
                            grid_r[ijk+ng1]==0 ||
                            grid_r[ijk-ng2]==0 ||
                            grid_r[ijk+ng2]==0) {
                        grid_r[ijk]=surface;
                    }
                    //*/
                }
            }
        }

        /*
            for (int i=ng1/2-1; i<ng1/2; i++) {
                for (int j=0; j<ng1; j++) {
                    printf("[");
                    for (int k=0; k<ng1; k++) {
                        const int ijk = i*ng2+j*ng1+k;
                        if (grid_r[ijk] <0) {
                            printf("* ");
                        } else if(grid_r[ijk] > 0) {
                            printf("%1.0f ",grid_r[ijk]);
                        } else {
                            printf("  ");
                        }
                    }
                    printf("]");
                    printf("\n");
                }
                printf("---ligand real %d/%d --------\n",i+1,ng1);
            }
        //*/

    } else {
//#else //NEWVOXEL ####################################################
// megadock 2.5.3 voxel

        for( int l = 0 ; l < na ; l++ ) {
            const int lc = ng1 * l;

            for( int i = 0 ; i < ng1 ; i++ ) {
                if( xd[lc+i] > radius_core2[l] ) continue;
                const int ib = ng2 * i;

                for( int j = 0 ; j < ng1 ; j++ ) {
                    if( yd[lc+j] > radius_core2[l] ) continue;
                    const float d2 = xd[lc+i] + yd[lc+j];
                    const int   jb = ib + ng1 * j;

                    for( int k = 0 ; k < ng1 ; k++ ) {
                        const float   d3 = d2 + zd[lc+k];
                        const int kb = jb + k;

                        if( d3 < distance3[kb] ) {
                            distance3[kb] = d3;
                            nearesta[kb] = l;
                        }
                    }
                }
            }
        }

        for( int i = 0 ; i < ng3 ; i++ ) {
            if( nearesta[i] == -99 ) continue;

            if( distance3[i] <= radius_core2[nearesta[i]] ) {
                grid_r[i] = delta;
            }
        }

        int   nsurf = 0;

        for( int i = 1 ; i < ng1-1 ; i++ ) {
            const int ib = ng2 * i;

            for( int j = 1 ; j < ng1-1 ; j++ ) {
                const int jb = ib + ng1 * j;

                for( int k = 1 ; k < ng1-1 ; k++ ) {
                    const int kb = jb + k;

                    if( grid_r[kb] == delta ) {
                        if( grid_r[kb-ng2] == 0.0 || grid_r[kb+ng2] == 0.0 ||
                                grid_r[kb-ng1] == 0.0 || grid_r[kb+ng1] == 0.0 ||
                                grid_r[kb-1  ] == 0.0 || grid_r[kb+1  ] == 0.0 ) {
                            grid_r[kb] = -9999.0;

                            int l;

                            for( l = 0 ; l < nsurf ; l++ ) {
                                if( surf_atom[l] == nearesta[kb] ) {
                                    break;
                                }
                            }

                            if( l >= nsurf ) {
                                surf_atom[nsurf++] = nearesta[kb];
                            }
                        }
                    }
                }
            }
        }

        for( int i = ng2 ; i < ng3-ng2 ; i++ ) {
            if( grid_r[i] == -9999.0 ) {
                grid_r[i] = 0.0;
            }
        }

        for( int l = 0 ; l < nsurf ; l++ ) {
            const int lc = ng1 * surf_atom[l];

            for( int i = 0 ; i < ng1 ; i++ ) {
                if( xd[lc+i] > radius_surf2[surf_atom[l]] ) continue;
                const int ib = ng2 * i;

                for( int j = 0 ; j < ng1 ; j++ ) {
                    if( yd[lc+j] > radius_surf2[surf_atom[l]] ) continue;
                    const float d2 = xd[lc+i] + yd[lc+j];
                    const int   jb = ib + ng1 * j;

                    for( int k = 0 ; k < ng1 ; k++ ) {
                        const float d3 = d2 + zd[lc+k];

                        if( d3 <= radius_surf2[surf_atom[l]] ) {
                            grid_r[jb+k] = 1.0;
                        }
                    }
                }
            }
        }
    }// megadock 2.5.3 voxel

    /*
    //for (int i=0; i<ng1; i++) {
    for (int i=ng1/2-1; i<ng1/2; i++) {
        for (int j=0; j<ng1; j++) {
            printf("[");
            for (int k=0; k<ng1; k++) {
                const int ijk = i*ng2+j*ng1+k;
                //printf("%.0f ",grid_r[ijk]);continue;
                if (grid_r[ijk] == delta) {
                    printf("* ");
                } else if(grid_r[ijk] != 0) {
                    printf("%2.0f",grid_r[ijk]);
                } else {
                    printf("  ");
                }
            }
            printf("]");
            printf("\n");
        }
        printf("---ligand real %d/%d --------\n",i+1,ng1);
    }
    //*/


    return;
}
//============================================================================//
void Ligand::electro(const float &beta,const float &eratio,
                     const int &num_grid,float *grid_coord,float *mol_coord,
                     int *iwork,float *fwork,const int &old_voxel_flag)
//============================================================================//
{
    const int ng1 = num_grid;
    const int ng2 = ng1 * ng1;
    const int ng3 = ng1 * ng1 * ng1;
    const int na = num_atoms();
    const int nag = num_atoms() * ng1;

    float     *grid_i;
    grid_i    = &fwork[ng3];

    if(old_voxel_flag==0) {
    	//*
        //const float grid_width = grid_coord[1] - grid_coord[0];
        const float grid_width = 1.2;
        const float pad = (ng1 * grid_width / 2);

        for( int l = 0 ; l < na ; l++ ) { //for each atom, find nearest grid(i,j,k)
            //if(fabs(_Charge[l]) < 0.0001) continue;
            const int l3 = l * 3;
            const int   i = (mol_coord[l3  ] + pad) / grid_width;
            const int   j = (mol_coord[l3+1] + pad) / grid_width;
            const int   k = (mol_coord[l3+2] + pad) / grid_width;
            
            //printf(" [%5d] [x:%12.8f,y:%12.8f,z:%12.8f] [pad:%6.3f], [%3d,%3d,%3d] \n",l,mol_coord[l3  ],mol_coord[l3+1],mol_coord[l3+2],pad,i,j,k);
            //printf(" [%5d] [x:%8.0f,y:%8.0f,z:%8.0f] [pad:%6.3f], [%3d,%3d,%3d] \n",l,mol_coord[l3  ],mol_coord[l3+1],mol_coord[l3+2],pad,i,j,k);
            /*
            if(grid_i[i*ng2+j*ng1+k]!=0){
                printf(" conflict: [%d,%d,%d] (%f > %f)\n",i,j,k,grid_i[i*ng2+j*ng1+k],_Charge[l]);
            }
            //*/
                        
            grid_i[i*ng2+j*ng1+k] += _Charge[l];
        }
        //for(int i=0;i<ng3;i++) if(grid_i[i]!=0.0) printf(" [%03d,%03d,%03d] :  %6.3f\n",i/ng2,i/ng1%ng1,i%ng1,grid_i[i]);
        //for(int i=0;i<ng3;i++) if(grid_i[i]!=0.0) printf(" [%03d,%03d,%03d] :  %6.3f\n",i/ng2,(i/ng1)%ng1,i%ng1,grid_i[i]);
        //*/
    } else {
//#else //NEWVOXEL_ELEC ####################################################
// megadock 2.5.3 voxel

        int       *ijk;
        const float   *xd, *yd, *zd;

        ijk       = &iwork[ng3*2+na];
        xd        = &fwork[ng3*3+na*3];
        yd        = &fwork[ng3*3+na*3+nag];
        zd        = &fwork[ng3*3+na*3+nag*2];

        //modified by t.ishida at 2011/4/15
        const float           grid_width=grid_coord[1]-grid_coord[0];
        const float           max_range=grid_width*1.5;
        const float           voxel_range = grid_width/2.0;

        for( int l = 0 ; l < na ; l++ ) {
            const int lc = ng1 * l;
            float distancea = max_range;

            for( int i = 0 ; i < ng1 ; i++ ) {
                if( xd[lc+i] > voxel_range ) continue;

                for( int j = 0 ; j < ng1 ; j++ ) {
                    if( yd[lc+j] > voxel_range ) continue;
                    const float d2 = xd[lc+i] + yd[lc+j];

                    for( int k = 0 ; k < ng1 ; k++ ) {
                        const float d3 = d2 + zd[lc+k];

                        if( d3 < distancea ) {
                            distancea   = d3;
                            ijk[3*l  ]  = i;
                            ijk[3*l+1]  = j;
                            ijk[3*l+2]  = k;
                        }
                    }
                }
            }
        }

        for( int l = 0 ; l < na ; l++ ) {
            const int   i = ijk[3*l  ];
            const int   j = ijk[3*l+1];
            const int   k = ijk[3*l+2];

            grid_i[i*ng2+j*ng1+k] = _Charge[l];
        }

    }// megadock 2.5.3 voxel

    /*
    //for (int i=0; i<ng1; i++) {
    for (int i=ng1/2-1; i<ng1/2; i++) {
        for (int j=0; j<ng1; j++) {
            printf("[");
            for (int k=0; k<ng1; k++) {
                const int ijk = i*ng2+j*ng1+k;
                //printf("%.0f ",grid_i[ijk]);continue;
                if (grid_i[ijk] == 0) {
                    printf("     ");
                } else {
                    //printf("* ");continue;
                    printf(" %2.2f",grid_i[ijk]);continue;
                    printf("  ");
                }
            }
            printf("]");
            printf("\n");
        }
        printf("---ligand elec %d/%d --------\n",i+1,ng1);
    }
    //*/


    return;
}

//============================================================================//
void Ligand::precalc(const int &num_grid,int *nearesta,float *rjudge2,
                     float *radius_core2,float *radius_surf2)
//============================================================================//
{
    const float   rcore2 = 1.5;           // ZDOCK parameter
    const float   rsurf2 = 1.0;           // ZDOCK parameter

    const int na  = num_atoms();
    const int ng1 = num_grid;
    const int ng3 = ng1 * ng1 * ng1;

    for( int i = 0 ; i < na ; i++ ) {
        radius_core2[i] = _Radius[i] * _Radius[i] * rcore2;
        radius_surf2[i] = _Radius[i] * _Radius[i] * rsurf2;
    }

    for( int i = 0 ; i < ng3 ; i++ ) {
        nearesta[i] = -99;
    }

    return;
}


/*
//============================================================================//
void Ligand::rpsc(const int &num_grid,float *grid_coord,float *mol_coord,
                  int *iwork,float *fwork, const float &param_rec_core, const float &param_lig_core)
//============================================================================//
{
    //    const float delta = 2.0;          // MEGADOCK parameter
    const float delta = param_lig_core;       // Tuning Parameter

    int       *nearesta;
    int       *surf_atom;
    float     *grid_r, *grid_i;
    float     *rjudge2, *radius_core2, *radius_surf2;
    float     *distance3;
    float     *xd, *yd, *zd;

    const int na  = num_atoms();
    const int nag = num_atoms() * num_grid;
    const int ng1 = num_grid;
    const int ng2 = num_grid * num_grid;
    const int ng3 = num_grid * num_grid * num_grid;

    nearesta  = iwork;
    surf_atom = &iwork[ng3];
    grid_r    = fwork;
    grid_i    = &fwork[ng3];
    rjudge2   = &fwork[ng3*2];
    radius_core2  = &fwork[ng3*2+na];
    radius_surf2  = &fwork[ng3*2+na*2];
    distance3     = &fwork[ng3*2+na*3];
    xd        = &fwork[ng3*3+na*3];
    yd        = &fwork[ng3*3+na*3+nag];
    zd        = &fwork[ng3*3+na*3+nag*2];

    precalc(num_grid,nearesta,rjudge2,radius_core2,radius_surf2);
    voxel_init(num_grid,grid_coord,mol_coord,grid_r,grid_i,distance3,xd,yd,zd);


#ifdef NEWVOXEL


#else //NEWVOXEL ####################################################
// megadock 2.5.3 voxel

    for( int l = 0 ; l < na ; l++ ) {
        const int lc = num_grid * l;

        for( int i = 0 ; i < num_grid ; i++ ) {
            if( xd[lc+i] > radius_core2[l] ) continue;
            const int ib = ng2 * i;

            for( int j = 0 ; j < num_grid ; j++ ) {
                if( yd[lc+j] > radius_core2[l] ) continue;
                const float d2 = xd[lc+i] + yd[lc+j];
                const int   jb = ib + num_grid * j;

                for( int k = 0 ; k < num_grid ; k++ ) {
                    const float   d3 = d2 + zd[lc+k];
                    const int kb = jb + k;

                    if( d3 < distance3[kb] ) {
                        distance3[kb] = d3;
                        nearesta[kb] = l;
                    }
                }
            }
        }
    }

    for( int i = 0 ; i < ng3 ; i++ ) {
        if( nearesta[i] == -99 ) continue;

        if( distance3[i] <= radius_core2[nearesta[i]] ) {
            grid_r[i] = delta;
        }
    }

    int   nsurf = 0;

    for( int i = 1 ; i < num_grid-1 ; i++ ) {
        const int ib = ng2 * i;

        for( int j = 1 ; j < num_grid-1 ; j++ ) {
            const int jb = ib + num_grid * j;

            for( int k = 1 ; k < num_grid-1 ; k++ ) {
                const int kb = jb + k;

                if( grid_r[kb] == delta ) {
                    if( grid_r[kb-ng2] == 0.0 || grid_r[kb+ng2] == 0.0 ||
                            grid_r[kb-ng1] == 0.0 || grid_r[kb+ng1] == 0.0 ||
                            grid_r[kb-1  ] == 0.0 || grid_r[kb+1  ] == 0.0 ) {
                        grid_r[kb] = -9999.0;

                        int l;

                        for( l = 0 ; l < nsurf ; l++ ) {
                            if( surf_atom[l] == nearesta[kb] ) {
                                break;
                            }
                        }

                        if( l >= nsurf ) {
                            surf_atom[nsurf++] = nearesta[kb];
                        }
                    }
                }
            }
        }
    }

    for( int i = ng2 ; i < ng3-ng2 ; i++ ) {
        if( grid_r[i] == -9999.0 ) {
            grid_r[i] = 0.0;
        }
    }

    for( int l = 0 ; l < nsurf ; l++ ) {
        const int lc = num_grid * surf_atom[l];

        for( int i = 0 ; i < num_grid ; i++ ) {
            if( xd[lc+i] > radius_surf2[surf_atom[l]] ) continue;
            const int ib = ng2 * i;

            for( int j = 0 ; j < num_grid ; j++ ) {
                if( yd[lc+j] > radius_surf2[surf_atom[l]] ) continue;
                const float d2 = xd[lc+i] + yd[lc+j];
                const int   jb = ib + num_grid * j;

                for( int k = 0 ; k < num_grid ; k++ ) {
                    const float d3 = d2 + zd[lc+k];

                    if( d3 <= radius_surf2[surf_atom[l]] ) {
                        grid_r[jb+k] = 1.0;
                    }
                }
            }
        }
    }

#endif // megadock 2.5.3 voxel


    return;
}
//*/